@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: session.h 12060 2011-03-01 15:23:22Z jordan $
+ * $Id: session.h 13499 2012-09-18 00:58:10Z livings124 $
  */
 
 #ifndef __TRANSMISSION__
@@ -27,36 +27,48 @@
  #endif
 #endif
 
+#include "bandwidth.h"
 #include "bencode.h"
 #include "bitfield.h"
 #include "utils.h"
 
 typedef enum { TR_NET_OK, TR_NET_ERROR, TR_NET_WAIT } tr_tristate_t;
 
-uint8_t*       tr_peerIdNew( void );
+typedef enum {
+    TR_AUTO_SWITCH_UNUSED,
+    TR_AUTO_SWITCH_ON,
+    TR_AUTO_SWITCH_OFF,
+} tr_auto_switch_state_t;
 
-const uint8_t* tr_getPeerId( void );
+enum
+{
+  PEER_ID_LEN = 20
+};
+
+void tr_peerIdInit( uint8_t * setme );
 
 struct event_base;
+struct evdns_base;
+
 struct tr_address;
 struct tr_announcer;
-struct tr_bandwidth;
+struct tr_announcer_udp;
 struct tr_bindsockets;
 struct tr_cache;
 struct tr_fdInfo;
 
-typedef void ( tr_web_config_func )( tr_session * session, void * curl_pointer, const char * url );
+typedef void ( tr_web_config_func )( tr_session * session, void * curl_pointer, const char * url, void * user_data );
 
 struct tr_turtle_info
 {
     /* TR_UP and TR_DOWN speed limits */
-    int speedLimit_Bps[2];
+    unsigned int speedLimit_Bps[2];
 
     /* is turtle mode on right now? */
-    tr_bool isEnabled;
+    bool isEnabled;
 
     /* does turtle mode turn itself on and off at given times? */
-    tr_bool isClockEnabled;
+    bool isClockEnabled;
 
     /* when clock mode is on, minutes after midnight to turn on turtle mode */
     int beginMinute;
@@ -75,39 +87,48 @@ struct tr_turtle_info
 
     /* the callback's changedByUser argument.
      * indicates whether the change came from the user or from the clock. */
-    tr_bool changedByUser;
+    bool changedByUser;
 
     /* bitfield of all the minutes in a week.
      * Each bit's value indicates whether the scheduler wants turtle
      * limits on or off at that given minute in the week. */
     tr_bitfield minutes;
+
+    /* recent action that was done by turtle's automatic switch */
+    tr_auto_switch_state_t autoTurtleState;
 };
 
 /** @brief handle to an active libtransmission session */
 struct tr_session
 {
-    tr_bool                      isPortRandom;
-    tr_bool                      isPexEnabled;
-    tr_bool                      isDHTEnabled;
-    tr_bool                      isLPDEnabled;
-    tr_bool                      isBlocklistEnabled;
-    tr_bool                      isPrefetchEnabled;
-    tr_bool                      isTorrentDoneScriptEnabled;
-    tr_bool                      isClosed;
-    tr_bool                      useLazyBitfield;
-    tr_bool                      isIncompleteFileNamingEnabled;
-    tr_bool                      isRatioLimited;
-    tr_bool                      isIdleLimited;
-    tr_bool                      isIncompleteDirEnabled;
-    tr_bool                      pauseAddedTorrent;
-    tr_bool                      deleteSourceTorrent;
+    bool                         isPortRandom;
+    bool                         isPexEnabled;
+    bool                         isDHTEnabled;
+    bool                         isUTPEnabled;
+    bool                         isLPDEnabled;
+    bool                         isBlocklistEnabled;
+    bool                         isPrefetchEnabled;
+    bool                         isTorrentDoneScriptEnabled;
+    bool                         isClosed;
+    bool                         isIncompleteFileNamingEnabled;
+    bool                         isRatioLimited;
+    bool                         isIdleLimited;
+    bool                         isIncompleteDirEnabled;
+    bool                         pauseAddedTorrent;
+    bool                         deleteSourceTorrent;
+    bool                         scrapePausedTorrents;
 
     tr_benc                      removedTorrents;
 
+    bool                         stalledEnabled;
+    bool                         queueEnabled[2];
+    int                          queueSize[2];
+    int                          queueStalledMinutes;
+
     int                          umask;
 
-    int                          speedLimit_Bps[2];
-    tr_bool                      speedLimitEnabled[2];
+    unsigned int                 speedLimit_Bps[2];
+    bool                         speedLimitEnabled[2];
 
     struct tr_turtle_info        turtle;
 
@@ -120,8 +141,10 @@ struct tr_session
     tr_preallocation_mode        preallocationMode;
 
     struct event_base          * event_base;
+    struct evdns_base          * evdns_base;
     struct tr_event_handle     * events;
 
+    uint16_t                     peerLimit;
     uint16_t                     peerLimitPerTorrent;
 
     int                          uploadSlotsPerTorrent;
@@ -182,6 +205,7 @@ struct tr_session
     struct tr_stats_handle     * sessionStats;
 
     struct tr_announcer        * announcer;
+    struct tr_announcer_udp    * announcer_udp;
 
     tr_benc                    * metainfoLookup;
 
@@ -189,22 +213,16 @@ struct tr_session
     struct event               * saveTimer;
 
     /* monitors the "global pool" speeds */
-    struct tr_bandwidth        * bandwidth;
+    struct tr_bandwidth          bandwidth;
 
-    double                       desiredRatio;
+    float                        desiredRatio;
 
     uint16_t                     idleLimitMinutes;
 
     struct tr_bindinfo         * public_ipv4;
     struct tr_bindinfo         * public_ipv6;
 
-    /* a page-aligned buffer for use by the libtransmission thread.
-     * @see SESSION_BUFFER_SIZE */
-    void * buffer;
-
-    tr_bool bufferInUse;
-
-    tr_web_config_func          * curl_easy_config_func;
+    uint8_t peer_id[PEER_ID_LEN+1];
 };
 
 static inline tr_port
@@ -213,9 +231,15 @@ tr_sessionGetPublicPeerPort( const tr_session * session )
     return session->public_peer_port;
 }
 
-tr_bool      tr_sessionAllowsDHT( const tr_session * session );
+static inline const uint8_t*
+tr_getPeerId( tr_session * session )
+{
+    return session->peer_id;
+}
 
-tr_bool      tr_sessionAllowsLPD( const tr_session * session );
+bool         tr_sessionAllowsDHT( const tr_session * session );
+
+bool         tr_sessionAllowsLPD( const tr_session * session );
 
 const char * tr_sessionFindTorrentFile( const tr_session * session,
                                         const char *       hashString );
@@ -224,18 +248,18 @@ void         tr_sessionSetTorrentFile( tr_session * session,
                                        const char * hashString,
                                        const char * filename );
 
-tr_bool      tr_sessionIsAddressBlocked( const tr_session        * session,
+bool         tr_sessionIsAddressBlocked( const tr_session        * session,
                                          const struct tr_address * addr );
 
 void         tr_sessionLock( tr_session * );
 
 void         tr_sessionUnlock( tr_session * );
 
-tr_bool      tr_sessionIsLocked( const tr_session * );
+bool         tr_sessionIsLocked( const tr_session * );
 
 const struct tr_address*  tr_sessionGetPublicAddress( const tr_session  * session,
                                                       int                 tr_af_type,
-                                                      tr_bool           * is_default_value );
+                                                      bool              * is_default_value );
 
 
 struct tr_bindsockets * tr_sessionGetBindSockets( tr_session * );
@@ -245,35 +269,28 @@ int tr_sessionCountTorrents( const tr_session * session );
 enum
 {
     SESSION_MAGIC_NUMBER = 3845,
-
-    /* @see tr_session.buffer */
-    SESSION_BUFFER_SIZE = (16*1024)
 };
 
-void* tr_sessionGetBuffer( tr_session * session );
-
-void tr_sessionReleaseBuffer( tr_session * session );
-
-static inline tr_bool tr_isSession( const tr_session * session )
+static inline bool tr_isSession( const tr_session * session )
 {
     return ( session != NULL ) && ( session->magicNumber == SESSION_MAGIC_NUMBER );
 }
 
-static inline tr_bool tr_isPreallocationMode( tr_preallocation_mode m  )
+static inline bool tr_isPreallocationMode( tr_preallocation_mode m  )
 {
     return ( m == TR_PREALLOCATE_NONE )
         || ( m == TR_PREALLOCATE_SPARSE )
         || ( m == TR_PREALLOCATE_FULL );
 }
 
-static inline tr_bool tr_isEncryptionMode( tr_encryption_mode m )
+static inline bool tr_isEncryptionMode( tr_encryption_mode m )
 {
     return ( m == TR_CLEAR_PREFERRED )
         || ( m == TR_ENCRYPTION_PREFERRED )
         || ( m == TR_ENCRYPTION_REQUIRED );
 }
 
-static inline tr_bool tr_isPriority( tr_priority_t p )
+static inline bool tr_isPriority( tr_priority_t p )
 {
     return ( p == TR_PRI_LOW )
         || ( p == TR_PRI_NORMAL )
@@ -284,25 +301,35 @@ static inline tr_bool tr_isPriority( tr_priority_t p )
 ****
 ***/
 
-static inline unsigned int toSpeedBytes ( unsigned int KBps ) { return KBps * tr_speed_K; }
-static inline double       toSpeedKBps  ( unsigned int Bps )  { return Bps / (double)tr_speed_K; }
+static inline unsigned int
+toSpeedBytes ( unsigned int KBps ) { return KBps * tr_speed_K; }
+static inline double
+toSpeedKBps  ( unsigned int Bps )  { return Bps / (double)tr_speed_K; }
 
-static inline uint64_t toMemBytes ( unsigned int MB ) { uint64_t B = tr_mem_K * tr_mem_K; B *= MB; return B; }
-static inline int      toMemMB    ( uint64_t B )      { return B / ( tr_mem_K * tr_mem_K ); }
+static inline uint64_t
+toMemBytes ( unsigned int MB ) { uint64_t B = tr_mem_K * tr_mem_K; B *= MB; return B; }
+static inline int
+toMemMB    ( uint64_t B )      { return B / ( tr_mem_K * tr_mem_K ); }
 
 /**
 **/
 
-int  tr_sessionGetSpeedLimit_Bps( const tr_session *, tr_direction );
-int  tr_sessionGetAltSpeed_Bps  ( const tr_session *, tr_direction );
-int  tr_sessionGetRawSpeed_Bps  ( const tr_session *, tr_direction );
-int  tr_sessionGetPieceSpeed_Bps( const tr_session *, tr_direction );
+unsigned int  tr_sessionGetSpeedLimit_Bps( const tr_session *, tr_direction );
+unsigned int  tr_sessionGetAltSpeed_Bps  ( const tr_session *, tr_direction );
+unsigned int  tr_sessionGetRawSpeed_Bps  ( const tr_session *, tr_direction );
+unsigned int  tr_sessionGetPieceSpeed_Bps( const tr_session *, tr_direction );
 
-void tr_sessionSetSpeedLimit_Bps( tr_session *, tr_direction, int Bps );
-void tr_sessionSetAltSpeed_Bps  ( tr_session *, tr_direction, int Bps );
+void tr_sessionSetSpeedLimit_Bps( tr_session *, tr_direction, unsigned int Bps );
+void tr_sessionSetAltSpeed_Bps  ( tr_session *, tr_direction, unsigned int Bps );
+
+bool  tr_sessionGetActiveSpeedLimit_Bps( const tr_session  * session,
+                                         tr_direction        dir,
+                                         unsigned int      * setme );
+
+tr_torrent * tr_sessionGetNextQueuedSeed( tr_session * session );
+tr_torrent * tr_sessionGetNextQueuedTorrent( tr_session * session, tr_direction );
+
+int tr_sessionCountQueueFreeSlots( tr_session * session, tr_direction );
 
-tr_bool  tr_sessionGetActiveSpeedLimit_Bps( const tr_session  * session,
-                                            tr_direction        dir,
-                                            int               * setme );
 
 #endif
