@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.c 13362 2012-07-01 03:05:36Z jordan $
+ * $Id: torrent.c 13470 2012-09-06 03:23:50Z livings124 $
  */
 
 #include <signal.h> /* signal() */
@@ -1145,16 +1145,23 @@ tr_torrentIsStalled( const tr_torrent * tor )
 static double
 getVerifyProgress( const tr_torrent * tor )
 {
-    tr_piece_index_t i, n;
-    tr_piece_index_t checked = 0;
+    double d = 0;
 
     assert( tr_isTorrent( tor ) );
 
-    for( i=0, n=tor->info.pieceCount; i!=n; ++i )
-        if( tor->info.pieces[i].timeChecked )
-            ++checked;
+    if( tr_torrentHasMetadata( tor ) )
+    {
+        tr_piece_index_t i, n;
+        tr_piece_index_t checked = 0;
 
-    return checked / (double)tor->info.pieceCount;
+        for( i=0, n=tor->info.pieceCount; i!=n; ++i )
+            if( tor->info.pieces[i].timeChecked )
+                ++checked;
+
+        d = checked / (double)tor->info.pieceCount;
+    }
+
+    return d;
 }
 
 const tr_stat *
