@@ -7,7 +7,7 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: mainwin.cc 11744 2011-01-21 21:51:50Z jordan $
+ * $Id: mainwin.cc 13446 2012-08-19 00:01:54Z jordan $
  */
 
 #include <cassert>
@@ -46,6 +46,27 @@
 
 #define PREFS_KEY "prefs-key";
 
+
+/**
+ * This is a proxy-style for that forces it to be always disabled.
+ * We use this to make our torrent list view behave consistently on
+ * both GTK and Qt implementations.
+ */
+class ListViewProxyStyle: public QProxyStyle
+{
+    public:
+        int styleHint(StyleHint hint,
+                      const QStyleOption *option = 0,
+                      const QWidget *widget = 0,
+                      QStyleHintReturn *returnData = 0) const
+        {
+            if (hint == QStyle::SH_ItemView_ActivateItemOnSingleClick)
+                return 0;
+            return QProxyStyle::styleHint(hint, option, widget, returnData);
+        }
+};
+
+
 QIcon
 TrMainWindow :: getStockIcon( const QString& name, int fallback )
 {
@@ -90,7 +111,9 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     mySpeedModeOnIcon( ":/icons/alt-limit-on.png" ),
     myLastSendTime( 0 ),
     myLastReadTime( 0 ),
-    myNetworkTimer( this )
+    myNetworkTimer( this ),
+    myRefreshTrayIconTimer( this ),
+    myRefreshActionSensitivityTimer( this )
 {
     setAcceptDrops( true );
 
@@ -104,12 +127,15 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     int i = style->pixelMetric( QStyle::PM_SmallIconSize, 0, this );
     const QSize smallIconSize( i, i );
 
+    ui.listView->setStyle(new ListViewProxyStyle);
+
     // icons
     ui.action_OpenFile->setIcon( getStockIcon( "folder-open", QStyle::SP_DialogOpenButton ) );
     ui.action_New->setIcon( getStockIcon( "document-new", QStyle::SP_DesktopIcon ) );
     ui.action_Properties->setIcon( getStockIcon( "document-properties", QStyle::SP_DesktopIcon ) );
     ui.action_OpenFolder->setIcon( getStockIcon( "folder-open", QStyle::SP_DirOpenIcon ) );
     ui.action_Start->setIcon( getStockIcon( "media-playback-start", QStyle::SP_MediaPlay ) );
+    ui.action_StartNow->setIcon( getStockIcon( "media-playback-start", QStyle::SP_MediaPlay ) );
     ui.action_Announce->setIcon( getStockIcon( "network-transmit-receive" ) );
     ui.action_Pause->setIcon( getStockIcon( "media-playback-pause", QStyle::SP_MediaPause ) );
     ui.action_Remove->setIcon( getStockIcon( "list-remove", QStyle::SP_TrashIcon ) );
@@ -122,6 +148,10 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     ui.action_Preferences->setIcon( getStockIcon( "preferences-system" ) );
     ui.action_Contents->setIcon( getStockIcon( "help-contents", QStyle::SP_DialogHelpButton ) );
     ui.action_About->setIcon( getStockIcon( "help-about" ) );
+    ui.action_QueueMoveTop->setIcon( getStockIcon( "go-top" ) );
+    ui.action_QueueMoveUp->setIcon( getStockIcon( "go-up", QStyle::SP_ArrowUp ) );
+    ui.action_QueueMoveDown->setIcon( getStockIcon( "go-down", QStyle::SP_ArrowDown ) );
+    ui.action_QueueMoveBottom->setIcon( getStockIcon( "go-bottom" ) );
 
     // ui signals
     connect( ui.action_Toolbar, SIGNAL(toggled(bool)), this, SLOT(setToolbarVisible(bool)));
@@ -133,11 +163,17 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_SortByETA,      SIGNAL(toggled(bool)), this, SLOT(onSortByETAToggled(bool)));
     connect( ui.action_SortByName,     SIGNAL(toggled(bool)), this, SLOT(onSortByNameToggled(bool)));
     connect( ui.action_SortByProgress, SIGNAL(toggled(bool)), this, SLOT(onSortByProgressToggled(bool)));
+    connect( ui.action_SortByQueue,    SIGNAL(toggled(bool)), this, SLOT(onSortByQueueToggled(bool)));
     connect( ui.action_SortByRatio,    SIGNAL(toggled(bool)), this, SLOT(onSortByRatioToggled(bool)));
     connect( ui.action_SortBySize,     SIGNAL(toggled(bool)), this, SLOT(onSortBySizeToggled(bool)));
     connect( ui.action_SortByState,    SIGNAL(toggled(bool)), this, SLOT(onSortByStateToggled(bool)));
     connect( ui.action_ReverseSortOrder, SIGNAL(toggled(bool)), this, SLOT(setSortAscendingPref(bool)));
     connect( ui.action_Start, SIGNAL(triggered()), this, SLOT(startSelected()));
+    connect( ui.action_QueueMoveTop,    SIGNAL(triggered()), this, SLOT(queueMoveTop()));
+    connect( ui.action_QueueMoveUp,     SIGNAL(triggered()), this, SLOT(queueMoveUp()));
+    connect( ui.action_QueueMoveDown,   SIGNAL(triggered()), this, SLOT(queueMoveDown()));
+    connect( ui.action_QueueMoveBottom, SIGNAL(triggered()), this, SLOT(queueMoveBottom()));
+    connect( ui.action_StartNow, SIGNAL(triggered()), this, SLOT(startSelectedNow()));
     connect( ui.action_Pause, SIGNAL(triggered()), this, SLOT(pauseSelected()));
     connect( ui.action_Remove, SIGNAL(triggered()), this, SLOT(removeSelected()));
     connect( ui.action_Delete, SIGNAL(triggered()), this, SLOT(deleteSelected()));
@@ -158,39 +194,17 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_SetLocation, SIGNAL(triggered()), this, SLOT(setLocation()));
     connect( ui.action_Properties, SIGNAL(triggered()), this, SLOT(openProperties()));
     connect( ui.action_SessionDialog, SIGNAL(triggered()), mySessionDialog, SLOT(show()));
+
     connect( ui.listView, SIGNAL(activated(const QModelIndex&)), ui.action_Properties, SLOT(trigger()));
 
-    QAction * sep2 = new QAction( this );
-    sep2->setSeparator( true );
-    QAction * sep3 = new QAction( this );
-    sep3->setSeparator( true );
-
-    // context menu
-    QList<QAction*> actions;
-    actions << ui.action_Properties
-            << ui.action_OpenFolder
-            << sep2
-            << ui.action_Start
-            << ui.action_Announce
-            << ui.action_Pause
-            << ui.action_CopyMagnetToClipboard
-            << sep3
-            << ui.action_Verify
-            << ui.action_SetLocation
-            << sep
-            << ui.action_Remove
-            << ui.action_Delete;
-    addActions( actions );
-    setContextMenuPolicy( Qt::ActionsContextMenu );
-
     // signals
     connect( ui.action_SelectAll, SIGNAL(triggered()), ui.listView, SLOT(selectAll()));
     connect( ui.action_DeselectAll, SIGNAL(triggered()), ui.listView, SLOT(clearSelection()));
 
     connect( &myFilterModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(refreshVisibleCount()));
     connect( &myFilterModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(refreshVisibleCount()));
-    connect( &myFilterModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivity()));
-    connect( &myFilterModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivity()));
+    connect( &myFilterModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivitySoon()));
+    connect( &myFilterModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(refreshActionSensitivitySoon()));
 
     connect( ui.action_Quit, SIGNAL(triggered()), QCoreApplication::instance(), SLOT(quit()) );
 
@@ -199,10 +213,10 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( &myModel, SIGNAL(modelReset()), this, SLOT(onModelReset()));
     connect( &myModel, SIGNAL(rowsRemoved(const QModelIndex&,int,int)), this, SLOT(onModelReset()));
     connect( &myModel, SIGNAL(rowsInserted(const QModelIndex&,int,int)), this, SLOT(onModelReset()));
-    connect( &myModel, SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&)), this, SLOT(refreshTrayIcon()));
+    connect( &myModel, SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&)), this, SLOT(refreshTrayIconSoon()));
 
     ui.listView->setModel( &myFilterModel );
-    connect( ui.listView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&,const QItemSelection&)), this, SLOT(refreshActionSensitivity()));
+    connect( ui.listView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&,const QItemSelection&)), this, SLOT(refreshActionSensitivitySoon()));
 
     QActionGroup * actionGroup = new QActionGroup( this );
     actionGroup->addAction( ui.action_SortByActivity );
@@ -210,10 +224,15 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     actionGroup->addAction( ui.action_SortByETA );
     actionGroup->addAction( ui.action_SortByName );
     actionGroup->addAction( ui.action_SortByProgress );
+    actionGroup->addAction( ui.action_SortByQueue );
     actionGroup->addAction( ui.action_SortByRatio );
     actionGroup->addAction( ui.action_SortBySize );
     actionGroup->addAction( ui.action_SortByState );
 
+    myAltSpeedAction = new QAction( tr( "Speed Limits" ), this );
+    myAltSpeedAction->setIcon( myPrefs.get<bool>(Prefs::ALT_SPEED_LIMIT_ENABLED) ? mySpeedModeOnIcon : mySpeedModeOffIcon );
+    connect( myAltSpeedAction, SIGNAL(triggered()), this, SLOT(toggleSpeedMode()) );
+
     QMenu * menu = new QMenu( );
     menu->addAction( ui.action_OpenFile );
     menu->addAction( ui.action_AddURL );
@@ -224,6 +243,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     menu->addSeparator( );
     menu->addAction( ui.action_StartAll );
     menu->addAction( ui.action_PauseAll );
+    menu->addAction( myAltSpeedAction );
     menu->addSeparator( );
     menu->addAction( ui.action_Quit );
     myTrayIcon.setContextMenu( menu );
@@ -273,8 +293,12 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
         myNetworkTimer.start( 1000 );
     }
 
-    refreshActionSensitivity( );
-    refreshTrayIcon( );
+    connect( &myRefreshTrayIconTimer, SIGNAL(timeout()), this, SLOT(refreshTrayIcon()) );
+    connect( &myRefreshActionSensitivityTimer, SIGNAL(timeout()), this, SLOT(refreshActionSensitivity()) );
+
+
+    refreshActionSensitivitySoon( );
+    refreshTrayIconSoon( );
     refreshStatusBar( );
     refreshTitle( );
     refreshVisibleCount( );
@@ -316,9 +340,9 @@ TrMainWindow :: onModelReset( )
 {
     refreshTitle( );
     refreshVisibleCount( );
-    refreshActionSensitivity( );
+    refreshActionSensitivitySoon( );
     refreshStatusBar( );
-    refreshTrayIcon( );
+    refreshTrayIconSoon( );
 }
 
 /****
@@ -527,6 +551,7 @@ void TrMainWindow :: onSortByAgeToggled      ( bool b ) { if( b ) setSortPref( S
 void TrMainWindow :: onSortByETAToggled      ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_ETA );      }
 void TrMainWindow :: onSortByNameToggled     ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_NAME );     }
 void TrMainWindow :: onSortByProgressToggled ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_PROGRESS ); }
+void TrMainWindow :: onSortByQueueToggled    ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_QUEUE );    }
 void TrMainWindow :: onSortByRatioToggled    ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_RATIO );    }
 void TrMainWindow :: onSortBySizeToggled     ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_SIZE );     }
 void TrMainWindow :: onSortByStateToggled    ( bool b ) { if( b ) setSortPref( SortMode::SORT_BY_STATE );    }
@@ -583,13 +608,45 @@ TrMainWindow :: setLocation( )
     d->show( );
 }
 
+// Open Folder & select torrent's file or top folder
+void openSelect(const QString& path)
+{
+#if defined(Q_OS_WIN)
+    const QString explorer = "explorer";
+        QString param;
+        if (!QFileInfo(path).isDir())
+            param = QLatin1String("/select,");
+        param += QDir::toNativeSeparators(path);
+        QProcess::startDetached(explorer, QStringList(param));
+#elif defined(Q_OS_MAC)
+    QStringList scriptArgs;
+        scriptArgs << QLatin1String("-e")
+                   << QString::fromLatin1("tell application \"Finder\" to reveal POSIX file \"%1\"")
+                                         .arg(path);
+        QProcess::execute(QLatin1String("/usr/bin/osascript"), scriptArgs);
+        scriptArgs.clear();
+        scriptArgs << QLatin1String("-e")
+                   << QLatin1String("tell application \"Finder\" to activate");
+        QProcess::execute("/usr/bin/osascript", scriptArgs);
+#elif defined(Q_OS_UNIX)
+    QDesktopServices :: openUrl( QUrl::fromLocalFile( path ) );
+#endif
+}
+
 void
 TrMainWindow :: openFolder( )
 {
     const int torrentId( *getSelectedTorrents().begin() );
     const Torrent * tor( myModel.getTorrentFromId( torrentId ) );
     const QString path( tor->getPath( ) );
-    QDesktopServices :: openUrl( QUrl::fromLocalFile( path ) );
+    const FileList files = tor->files();
+    if (files.size() == 1)
+        openSelect( path + "/" + files.at(0).filename );
+    else {
+        QDir dir( path + "/" + files.at(0).filename );
+        dir.cdUp();
+        openSelect( dir.path() );
+    }
 }
 
 void
@@ -640,6 +697,15 @@ TrMainWindow :: refreshVisibleCount( )
     myVisibleCountLabel->setVisible( totalCount > 0 );
 }
 
+void
+TrMainWindow :: refreshTrayIconSoon( )
+{
+    if( !myRefreshTrayIconTimer.isActive( ) )
+    {
+        myRefreshTrayIconTimer.setSingleShot( true );
+        myRefreshTrayIconTimer.start( 100 );
+    }
+}
 void
 TrMainWindow :: refreshTrayIcon( )
 {
@@ -694,12 +760,25 @@ TrMainWindow :: refreshStatusBar( )
     myStatsLabel->setText( str );
 }
 
+
+
+void
+TrMainWindow :: refreshActionSensitivitySoon( )
+{
+    if( !myRefreshActionSensitivityTimer.isActive( ) )
+    {
+        myRefreshActionSensitivityTimer.setSingleShot( true );
+        myRefreshActionSensitivityTimer.start( 100 );
+    }
+}
 void
 TrMainWindow :: refreshActionSensitivity( )
 {
     int selected( 0 );
     int paused( 0 );
+    int queued( 0 );
     int selectedAndPaused( 0 );
+    int selectedAndQueued( 0 );
     int canAnnounce( 0 );
     const QAbstractItemModel * model( ui.listView->model( ) );
     const QItemSelectionModel * selectionModel( ui.listView->selectionModel( ) );
@@ -713,14 +792,13 @@ TrMainWindow :: refreshActionSensitivity( )
         if( tor ) {
             const bool isSelected( selectionModel->isSelected( modelIndex ) );
             const bool isPaused( tor->isPaused( ) );
-            if( isSelected )
-                ++selected;
-            if( isPaused )
-                ++ paused;
-            if( isSelected && isPaused )
-                ++selectedAndPaused;
-            if( tor->canManualAnnounce( ) )
-                ++canAnnounce;
+            const bool isQueued( tor->isQueued( ) );
+            if( isSelected ) ++selected;
+            if( isQueued ) ++queued;
+            if( isPaused ) ++ paused;
+            if( isSelected && isPaused ) ++selectedAndPaused;
+            if( isSelected && isQueued ) ++selectedAndQueued;
+            if( tor->canManualAnnounce( ) ) ++canAnnounce;
         }
     }
 
@@ -740,9 +818,15 @@ TrMainWindow :: refreshActionSensitivity( )
     ui.action_StartAll->setEnabled( paused > 0 );
     ui.action_PauseAll->setEnabled( paused < rowCount );
     ui.action_Start->setEnabled( selectedAndPaused > 0 );
+    ui.action_StartNow->setEnabled( selectedAndPaused + selectedAndQueued > 0 );
     ui.action_Pause->setEnabled( selectedAndPaused < selected );
     ui.action_Announce->setEnabled( selected > 0 && ( canAnnounce == selected ) );
 
+    ui.action_QueueMoveTop->setEnabled( haveSelection );
+    ui.action_QueueMoveUp->setEnabled( haveSelection );
+    ui.action_QueueMoveDown->setEnabled( haveSelection );
+    ui.action_QueueMoveBottom->setEnabled( haveSelection );
+
     if( myDetailsDialog )
         myDetailsDialog->setIds( getSelectedTorrents( ) );
 }
@@ -777,11 +861,36 @@ TrMainWindow :: startSelected( )
     mySession.startTorrents( getSelectedTorrents( ) );
 }
 void
+TrMainWindow :: startSelectedNow( )
+{
+    mySession.startTorrentsNow( getSelectedTorrents( ) );
+}
+void
 TrMainWindow :: pauseSelected( )
 {
     mySession.pauseTorrents( getSelectedTorrents( ) );
 }
 void
+TrMainWindow :: queueMoveTop( )
+{
+    mySession.queueMoveTop( getSelectedTorrents( ) );
+}
+void
+TrMainWindow :: queueMoveUp( )
+{
+    mySession.queueMoveUp( getSelectedTorrents( ) );
+}
+void
+TrMainWindow :: queueMoveDown( )
+{
+    mySession.queueMoveDown( getSelectedTorrents( ) );
+}
+void
+TrMainWindow :: queueMoveBottom( )
+{
+    mySession.queueMoveBottom( getSelectedTorrents( ) );
+}
+void
 TrMainWindow :: startAll( )
 {
     mySession.startTorrents( );
@@ -833,7 +942,11 @@ TrMainWindow :: setCompactView( bool visible )
 void
 TrMainWindow :: toggleSpeedMode( )
 {
+    bool mode;
+    
     myPrefs.toggleBool( Prefs :: ALT_SPEED_LIMIT_ENABLED );
+    mode = myPrefs.get<bool>( Prefs::ALT_SPEED_LIMIT_ENABLED );
+    myAltSpeedAction->setIcon( mode ? mySpeedModeOnIcon : mySpeedModeOffIcon );
 }
 void
 TrMainWindow :: setToolbarVisible( bool visible )
@@ -914,6 +1027,7 @@ TrMainWindow :: refreshPref( int key )
             ui.action_SortByETA->setChecked      ( i == SortMode::SORT_BY_ETA );
             ui.action_SortByName->setChecked     ( i == SortMode::SORT_BY_NAME );
             ui.action_SortByProgress->setChecked ( i == SortMode::SORT_BY_PROGRESS );
+            ui.action_SortByQueue->setChecked    ( i == SortMode::SORT_BY_QUEUE );
             ui.action_SortByRatio->setChecked    ( i == SortMode::SORT_BY_RATIO );
             ui.action_SortBySize->setChecked     ( i == SortMode::SORT_BY_SIZE );
             ui.action_SortByState->setChecked    ( i == SortMode::SORT_BY_STATE );
@@ -965,7 +1079,7 @@ TrMainWindow :: refreshPref( int key )
             b = myPrefs.getBool( key );
             ui.action_TrayIcon->setChecked( b );
             myTrayIcon.setVisible( b );
-            refreshTrayIcon( );
+            refreshTrayIconSoon( );
             break;
 
         case Prefs::COMPACT_VIEW: {
@@ -1064,11 +1178,12 @@ TrMainWindow :: openURL( QString url )
 {
     bool ok;
     const QString key = QInputDialog::getText( this,
-                                               tr( "Open URL or Magnet Link" ),
+                                               tr( "Open Link" ),
                                                tr( "Open URL or Magnet Link" ),
                                                QLineEdit::Normal,
                                                url,
-                                               &ok );
+                                               &ok,
+                                               Qt::WindowStaysOnTopHint );
     if( ok && !key.isEmpty( ) )
         mySession.addTorrent( key );
 }
@@ -1255,7 +1370,8 @@ TrMainWindow :: dragEnterEvent( QDragEnterEvent * event )
     const QMimeData * mime = event->mimeData( );
 
     if( mime->hasFormat("application/x-bittorrent")
-            || mime->text().trimmed().endsWith(".torrent", Qt::CaseInsensitive) )
+            || mime->text().trimmed().endsWith(".torrent", Qt::CaseInsensitive)
+            || mime->text().startsWith("magnet:", Qt::CaseInsensitive) )
         event->acceptProposedAction();
 }
 
@@ -1270,3 +1386,44 @@ TrMainWindow :: dropEvent( QDropEvent * event )
 
     dynamic_cast<MyApp*>(QApplication::instance())->addTorrent( key );
 }
+
+/***
+****
+***/
+
+void
+TrMainWindow :: contextMenuEvent( QContextMenuEvent * event )
+{
+    QMenu * menu = new QMenu( this );
+
+    menu->addAction( ui.action_Properties );
+    menu->addAction( ui.action_OpenFolder );
+
+    QAction * sep = new QAction( this );
+    sep->setSeparator( true );
+    menu->addAction( sep );
+    menu->addAction( ui.action_Start );
+    menu->addAction( ui.action_StartNow );
+    menu->addAction( ui.action_Announce );
+    QMenu * queueMenu = menu->addMenu( tr( "Queue" ) );
+        queueMenu->addAction( ui.action_QueueMoveTop );
+        queueMenu->addAction( ui.action_QueueMoveUp );
+        queueMenu->addAction( ui.action_QueueMoveDown );
+        queueMenu->addAction( ui.action_QueueMoveBottom );
+    menu->addAction( ui.action_Pause );
+
+    sep = new QAction( this );
+    sep->setSeparator( true );
+    menu->addAction( sep );
+    menu->addAction( ui.action_Verify );
+    menu->addAction( ui.action_SetLocation );
+    menu->addAction( ui.action_CopyMagnetToClipboard );
+
+    sep = new QAction( this );
+    sep->setSeparator( true );
+    menu->addAction( sep );
+    menu->addAction( ui.action_Remove );
+    menu->addAction( ui.action_Delete );
+
+    menu->popup( event->globalPos( ) );
+}
