@@ -53,6 +53,10 @@ Transmission.prototype =
 
 		$('#upload_confirm_button').click($.proxy(this.confirmUploadClicked,this));
 		$('#upload_cancel_button').click($.proxy(this.hideUploadDialog,this));
+
+		$('#move_confirm_button').click($.proxy(this.confirmMoveClicked,this));
+		$('#move_cancel_button').click($.proxy(this.hideMoveDialog,this));
+
 		$('#turtle-button').click($.proxy(this.toggleTurtleClicked,this));
 		$('#compact-button').click($.proxy(this.toggleCompactClicked,this));
 
@@ -111,6 +115,7 @@ Transmission.prototype =
 			var o = data['arguments'];
 			Prefs.getClutchPrefs(o);
 			this.updateGuiFromSession(o);
+			this.sessionProperties = o;
 		}, this, async);
 	},
 
@@ -175,6 +180,7 @@ Transmission.prototype =
 			context_pause_selected:       function() { tr.stopSelectedTorrents(); },
 			context_resume_selected:      function() { tr.startSelectedTorrents(false); },
 			context_resume_now_selected:  function() { tr.startSelectedTorrents(true); },
+			context_move:                 function() { tr.moveSelectedTorrents(false); },
 			context_remove:               function() { tr.removeSelectedTorrents(); },
 			context_removedata:           function() { tr.removeSelectedTorrentsAndData(); },
 			context_verify:               function() { tr.verifySelectedTorrents(); },
@@ -182,7 +188,9 @@ Transmission.prototype =
 			context_move_top:             function() { tr.moveTop(); },
 			context_move_up:              function() { tr.moveUp(); },
 			context_move_down:            function() { tr.moveDown(); },
-			context_move_bottom:          function() { tr.moveBottom(); }
+			context_move_bottom:          function() { tr.moveBottom(); },
+			context_select_all:           function() { tr.selectAll(); },
+			context_deselect_all:         function() { tr.deselectAll(); }
 		};
 
 		// Set up the context menu
@@ -254,7 +262,7 @@ Transmission.prototype =
 	},
 
 	seedRatioLimit: function() {
-		var p = this._prefs;
+		var p = this.sessionProperties;
 		if (p && p.seedRatioLimited)
 			return p.seedRatioLimit;
 		return -1;
@@ -497,7 +505,7 @@ Transmission.prototype =
 	drop: function(ev)
 	{
 		var i, uri, uris=null,
-		    types = ["text/uri-list", "text/plain"];
+		    types = ["text/uri-list", "text/plain"],
 		    paused = this.shouldAddedTorrentsStart();
 
 		if (!ev.dataTransfer || !ev.dataTransfer.types)
@@ -529,6 +537,16 @@ Transmission.prototype =
 		this.hideUploadDialog();
 	},
 
+	hideMoveDialog: function() {
+		$('#move_container').hide();
+		this.updateButtonStates();
+	},
+
+	confirmMoveClicked: function() {
+		this.moveSelectedTorrents(true);
+		this.hideUploadDialog();
+	},
+
 	removeClicked: function(ev) {
 		if (this.isButtonEnabled(ev)) {
 			this.removeSelectedTorrents();
@@ -679,6 +697,10 @@ Transmission.prototype =
 				this.setSortDirection(dir);
 				break;
 
+			case 'toggle_notifications':
+				Notifications && Notifications.toggle();
+				break;
+
 			default:
 				console.log('unhandled: ' + id);
 				break;
@@ -722,6 +744,16 @@ Transmission.prototype =
 				// do we need more info for this torrent?
 				if(!('name' in t.fields) || !('status' in t.fields))
 					needinfo.push(id);
+
+				t.notifyOnFieldChange('status', $.proxy(function (newValue, oldValue) {
+					if (oldValue === Torrent._StatusDownload && (newValue == Torrent._StatusSeed || newValue == Torrent._StatusSeedWait)) {
+						$(this).trigger('downloadComplete', [t]);
+					} else if (oldValue === Torrent._StatusSeed && newValue === Torrent._StatusStopped && t.isFinished()) {
+						$(this).trigger('seedingComplete', [t]);
+					} else {
+						$(this).trigger('statusChange', [t]);
+					}
+				}, this));
 			}
 		}
 
@@ -870,6 +902,34 @@ Transmission.prototype =
 		}
 	},
 
+	promptSetLocation: function(confirmed, torrents) {
+		if (! confirmed) {
+			var path;
+			if (torrents.length === 1) {
+				path = torrents[0].getDownloadDir();
+			} else {
+				path = $("#download-dir").val();
+			}
+			$('input#torrent_path').attr('value', path);
+			$('#move_container').show();
+			$('#torrent_path').focus();
+		} else {
+			var ids = this.getTorrentIds(torrents);
+			this.remote.moveTorrents(
+				ids, 
+				$("input#torrent_path").val(), 
+				this.refreshTorrents, 
+				this);
+			$('#move_container').hide();
+		}
+	},
+
+	moveSelectedTorrents: function(confirmed) {
+		var torrents = this.getSelectedTorrents();
+		if (torrents.length)
+			this.promptSetLocation(confirmed, torrents);
+	},
+
 	removeSelectedTorrents: function() {
 		var torrents = this.getSelectedTorrents();
 		if (torrents.length)
@@ -882,8 +942,7 @@ Transmission.prototype =
 			this.promptToRemoveTorrentsAndData(torrents);
 	},
 
-	promptToRemoveTorrents:function(torrents)
-	{
+	promptToRemoveTorrents: function(torrents) {
 		if (torrents.length === 1)
 		{
 			var torrent = torrents[0],
